@@ -1,5 +1,4 @@
-from quickstats._version import __version__
-from quickstats.decorators import *
+from ._version import __version__
 
 import os
 import pathlib
@@ -17,11 +16,11 @@ stylesheet_path = os.path.join(module_path, 'stylesheets')
 MAX_WORKERS = 8
 
 from quickstats.utils.io import VerbosePrint
-
 _PRINT_ = VerbosePrint("INFO")
 
 corelib_loaded = False
 
 from quickstats.main import *
+from quickstats.core import *
 
 root_version = get_root_version()
\ No newline at end of file
