@@ -1252,7 +1252,8 @@ class ExtendedModel(AbstractObject):
             else:
                 # check if the event with non-unity weight are from the ghost
                 y_with_ghost_removed = y[y > 1e-8]
-                if np.all(y_with_ghost_removed == 1.):
+                if (np.all(y_with_ghost_removed == 1.)) or \
+                   (len(np.unique(y_with_ghost_removed)) == 1):
                     binned = False
                     ghost  = True
                 elif len(x) == default_bins:
@@ -1262,10 +1263,12 @@ class ExtendedModel(AbstractObject):
                     nbins     = binnings[cat]['bins']
                     bin_range = binnings[cat]['bin_range']
                     correct_bins = get_bins_given_edges(bin_range[0], bin_range[1], nbins)
-                    issubset = array_issubset(correct_bins, x)
+                    correct_bins = np.around(correct_bins, 8)
+                    dataset_bins = np.around(x, 8)
+                    issubset = array_issubset(correct_bins, dataset_bins)
                     if issubset:
                         binned = True
-                        missing_bins   = np.setdiff1d(correct_bins, x)
+                        missing_bins   = np.setdiff1d(correct_bins, dataset_bins)
                         # fill missing bins with zero values
                         missing_values = np.zeros(missing_bins.shape[0])
                         x = np.concatenate([x, missing_bins])
@@ -1300,8 +1303,7 @@ class ExtendedModel(AbstractObject):
                 distributions[cat]['yerrlo'] = bin_errors['lo']
                 distributions[cat]['yerrhi'] = bin_errors['hi']
         return distributions
-    
-    @semistaticmethod
+
     def get_simul_pdf_distributions(self, pdf, observables, n_bins:Optional[Union[Dict, int]]=None):
         if not isinstance(pdf, ROOT.RooSimultaneous):
             raise ValueError("input pdf is not a simultaneous pdf")
