@@ -31,7 +31,8 @@ class NuisanceParameterHarmonizer(object):
         from quickstats.components import ExtendedModel
         nuis_names_collection = set()
         for ws_path in ws_paths:
-            nuis_names = ExtendedModel.get_nuisance_parameter_names(ws_path)
+            model = ExtendedModel(ws_path, data_name=None, verbosity="WARNING")
+            nuis_names = [np.GetName() for np in model.nuisance_parameters]
             nuis_names_collection |= set(nuis_names)
         return sorted(list(nuis_names_collection))
     
