@@ -17,17 +17,14 @@ import numpy as np
 import ROOT
 
 import quickstats
-from quickstats import semistaticmethod
-from quickstats.components import AbstractObject, ExtendedModel
+from quickstats import semistaticmethod, AbstractObject, GeneralEnum, Timer
+from quickstats.components import ExtendedModel
 from quickstats.utils.io import Verbosity
 from quickstats.utils.root_utils import load_macro, get_macro_dir, is_corrupt
 from quickstats.utils.common_utils import format_delimiter_enclosed_text
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.workspaces import XMLWSBase, WSObjectType, BracketType
-from quickstats.decorators import Timer
 
-from quickstats.utils.enums import GeneralEnum
-    
 class XMLWSCombiner(XMLWSBase):
     
     KEYWORDS = {
