@@ -13,16 +13,14 @@ import numpy as np
 import ROOT
 
 import quickstats
-from quickstats import semistaticmethod
-from quickstats.components import AbstractObject, ExtendedModel
+from quickstats import semistaticmethod, Timer, AbstractObject, GeneralEnum
+from quickstats.components import ExtendedModel
 from quickstats.utils.io import Verbosity
 from quickstats.utils.root_utils import load_macro, get_macro_dir
 from quickstats.utils.common_utils import format_delimiter_enclosed_text, remove_list_duplicates
 from quickstats.maths.numerics import is_float, pretty_value
 from quickstats.components.basics import WSArgument
 from quickstats.components.workspaces import XMLWSBase, WSObjectType, AsimovHandler
-from quickstats.decorators import Timer
-from quickstats.utils.enums import GeneralEnum
 
 class ModifierAction(GeneralEnum):
     NORMAL     = 0
