@@ -2,15 +2,14 @@ from typing import Dict, Union, List, Optional
 
 import numpy as np
 
-import ROOT
-
 from quickstats import semistaticmethod
 from quickstats.interface.cppyy.vectorize import as_np_array
 
 class RooDataSet:
     
     @staticmethod
-    def to_numpy(dataset:ROOT.RooDataSet):
+    def to_numpy(dataset:"ROOT.RooDataSet"):
+        import ROOT
         components = dataset.get()
         observables = ROOT.RooArgSet()
         category = None
@@ -39,9 +38,10 @@ class RooDataSet:
     
     @staticmethod
     def from_numpy(data:Dict[str, "numpy.ndarray"],
-                   variables:ROOT.RooArgSet,
+                   variables:"ROOT.RooArgSet",
                    name:str=None, title:str=None,
                    weight_name:str=None, clip_to_limits=True):
+        import ROOT
         if name is None:
             name = ""
         if title is None:
@@ -98,7 +98,7 @@ class RooDataSet:
         return dataset
     
     @semistaticmethod
-    def to_pandas(self, dataset:ROOT.RooDataSet):
+    def to_pandas(self, dataset:"ROOT.RooDataSet"):
         numpy_data = self.to_numpy(dataset, reduced=reduced)
         import pandas as pd
         df = pd.DataFrame(numpy_data)
@@ -106,6 +106,7 @@ class RooDataSet:
     
     @staticmethod
     def get_dataset_map(dataset_dict:Dict):
+        import ROOT
         dsmap = ROOT.std.map('string, RooDataSet*')()
         dsmap.keepalive = list()
         for c, d in dataset_dict.items():
@@ -114,7 +115,9 @@ class RooDataSet:
         return dsmap
     
     @staticmethod
-    def from_RooDataHist(source:ROOT.RooDataHist, pdf:ROOT.RooAbsPdf, name:Optional[str]=None):
+    def from_RooDataHist(source:"ROOT.RooDataHist", pdf:"ROOT.RooAbsPdf",
+                         name:Optional[str]=None):
+        import ROOT
         if name is None:
             name = source.GetName()
         parameters = source.get()
@@ -161,7 +164,7 @@ class RooDataSet:
         return dataset
 
     @staticmethod
-    def _get_cat_and_obs(variables:ROOT.RooArgSet):
+    def _get_cat_and_obs(variables:"ROOT.RooArgSet"):
         cat_variable = None
         observables = {}
         for v in variables:
@@ -179,10 +182,11 @@ class RooDataSet:
 
     @semistaticmethod
     def create_binned_category_dataset(self, data:Dict[str, "numpy.ndarray"],
-                                       pdf:ROOT.RooAbsPdf,
-                                       variables:ROOT.RooArgSet,
+                                       pdf:"ROOT.RooAbsPdf",
+                                       variables:"ROOT.RooArgSet",
                                        weight_name:str="weightVar",
                                        name:str=None, title:str=None):
+        import ROOT
         if name is None:
             name = ""
         if title is None:
@@ -219,8 +223,8 @@ class RooDataSet:
         return dataset
     
     @semistaticmethod
-    def to_category_data(self, dataset:ROOT.RooDataSet,
-                         pdf:ROOT.RooAbsPdf):
+    def to_category_data(self, dataset:"ROOT.RooDataSet",
+                         pdf:"ROOT.RooAbsPdf"):
         variables = dataset.get()
         cat_variable, observables = self._get_cat_and_obs(variables)
         n_cat = cat_variable.size()
