@@ -2,7 +2,9 @@ import quickstats
 
 from .color_schemes import *
 from .abstract_plot import AbstractPlot
+from .stat_plot_config import *
 from .hypotest_inverter_plot import HypoTestInverterPlot
+from .variable_distribution_plot import VariableDistributionPlot
 from .score_distribution_plot import score_distribution_plot, score_distribution_plot_2D, ScoreDistributionPlot
 from .test_statistic_distribution_plot import TestStatisticDistributionPlot
 from .general_1D_plot import General1DPlot
