@@ -46,6 +46,7 @@ class AbstractPlot:
         
         self.legend_data = {}
         self.legend_data_sec = {}
+        self.legend_data_ext = {}
         
         self.figure_index = figure_index
         
@@ -70,11 +71,17 @@ class AbstractPlot:
     def get_default_legend_order(self):
         return []
     
-    def update_legend_handles(self, handles:Dict, sec:bool=False):
-        if not sec:
-            legend_data = self.legend_data
+    def update_legend_handles(self, handles:Dict, sec:bool=False,
+                              idx:Optional[int]=None):
+        if idx is None:
+            if not sec:
+                legend_data = self.legend_data
+            else:
+                legend_data = self.legend_data_sec
         else:
-            legend_data = self.legend_data_sec
+            if idx not in self.legend_data_ext:
+                self.legend_data_ext[idx] = {}
+            legend_data = self.legend_data_ext[idx]
             
         for key in handles:
             handle = handles[key]
@@ -92,19 +99,34 @@ class AbstractPlot:
             else:
                 raise RuntimeError(f"the handle {handle} does not have an associated label")
 
-    def get_legend_handles_labels(self, sec:bool=False):
+    def get_legend_handles_labels(self, sec:bool=False,
+                                  idx:Optional[Union[int, List[int]]]=None):
         handles = []
         labels = []
-        if not sec:
-            legend_data = self.legend_data
+        if idx is None:
+            if not sec:
+                legend_data = self.legend_data
+            else:
+                legend_data = self.legend_data_sec        
+            for key in self.legend_order:
+                if key in legend_data:
+                    handle = legend_data[key]['handle']
+                    label = legend_data[key]['label']
+                    handles.append(handle)
+                    labels.append(label)
         else:
-            legend_data = self.legend_data_sec        
-        for key in self.legend_order:
-            if key in legend_data:
-                handle = legend_data[key]['handle']
-                label = legend_data[key]['label']
-                handles.append(handle)
-                labels.append(label)
+            if isinstance(idx, int):
+                indices = [idx]
+            else:
+                indices = idx
+            for index in indices:
+                legend_data = self.legend_data_ext[index]
+                for key in self.legend_order:
+                    if key in legend_data:
+                        handle = legend_data[key]['handle']
+                        label = legend_data[key]['label']
+                        handles.append(handle)
+                        labels.append(label)
         return handles, labels
     
     def draw_frame(self, frame_method=None, **kwargs):
