@@ -9,6 +9,11 @@ from quickstats.plots.color_schemes import QUICKSTATS_PALETTES
 from quickstats.plots import AbstractPlot
 from quickstats.plots.template import suggest_markersize, ratio_frames, centralize_axis, create_transform
 from quickstats.utils.common_utils import combine_dict
+from quickstats import GeneralEnum
+
+class ErrorDisplayFormat(GeneralEnum):
+    ERRORBAR = 0
+    FILL     = 1
 
 class PdfDistributionPlot(AbstractPlot):
     
@@ -30,6 +35,9 @@ class PdfDistributionPlot(AbstractPlot):
         },
         "legend": {
             "borderpad": 1
+        },
+        "ratio_frames": {
+            "height_ratios": (4, 1)
         }
     }
     
@@ -41,8 +49,9 @@ class PdfDistributionPlot(AbstractPlot):
         }        
     }
     
-    def __init__(self, collective_data, blind_range:Optional[List[float]]=None,
+    def __init__(self, collective_data:Dict, blind_range:Optional[List[float]]=None,
                  plot_options:Optional[Dict]=None,
+                 scale_map:Optional[Dict]=None,
                  label_map:Optional[Dict]=None,
                  color_cycle:Optional[Dict]=None,
                  styles:Optional[Union[Dict, str]]=None,
@@ -53,8 +62,8 @@ class PdfDistributionPlot(AbstractPlot):
                          styles=styles, figure_index=figure_index,
                          analysis_label_options=analysis_label_options,
                          config=config)
-        self.collective_data = collective_data
-        self.legend_order = list(collective_data.keys())
+        self.error_options = {}
+        self.set_data(collective_data)
         if plot_options is None:
             self.plot_options = {}
         else:
@@ -63,10 +72,20 @@ class PdfDistributionPlot(AbstractPlot):
             self.label_map = {}
         else:
             self.label_map = label_map
-
+        if scale_map is None:
+            self.scale_map = {}
+        else:
+            self.scale_map = scale_map
         self.colors = {}
         
         self.annotation = None
+        
+    def set_data(self, collective_data:Dict):
+        self.collective_data = collective_data
+        self.legend_order = list(collective_data.keys())
+        for name in self.collective_data:
+            if name not in self.error_options:
+                self.set_error_options(name)
 
     def get_error_data(self, data):
         xerr = data.get('xerr', None)
@@ -83,15 +102,15 @@ class PdfDistributionPlot(AbstractPlot):
             raise ValueError("invalid data: received both \"yerr\" and \"yerrlo\"/\"yerrhi\" attributes")
         error_data = {}
         if xerr is not None:
-            error_data['xerr'] = xerr
+            error_data['xerr'] = np.array(xerr)
         elif ((xerrlo is not None) or (xerrhi is not None)):
-            error_data['xerr'] = (xerrlo, xerrhi)
+            error_data['xerr'] = (np.array(xerrlo), np.array(xerrhi))
         else:
             error_data['xerr'] = None
         if yerr is not None:
-            error_data['yerr'] = yerr
+            error_data['yerr'] = np.array(yerr)
         elif ((yerrlo is not None) or (yerrhi is not None)):
-            error_data['yerr'] = (yerrlo, yerrhi)
+            error_data['yerr'] = (np.array(yerrlo), np.array(yerrhi))
         else:
             error_data['yerr'] = None
         return error_data
@@ -107,9 +126,13 @@ class PdfDistributionPlot(AbstractPlot):
         else:
             return err[mask]
         
-    def draw_single_data(self, ax, data, label:str, blind_range:Optional[List[float]]=None,
-                         plot_options:Optional[Dict]=None, show_errorbar:bool=True):
-        pdata = self.process_data(data, blind_range=blind_range)
+    def draw_single_data(self, ax, data, label:str, label_blind:Optional[str]=None,
+                         blind_range:Optional[List[float]]=None,
+                         plot_options:Optional[Dict]=None, show_error:bool=True,
+                         error_format:Union[ErrorDisplayFormat, str]="error_bar",
+                         error_styles:Optional[Dict]=None,
+                         scale_by:Optional[float]=None):
+        pdata = self.process_data(data, blind_range=blind_range, scale_by=scale_by)
         has_errorbar = not ((pdata['xerr'] is None) and (pdata['yerr'] is None))
         draw_blind = blind_range is not None
         if not has_errorbar:
@@ -127,44 +150,58 @@ class PdfDistributionPlot(AbstractPlot):
                                       pdata['y'][pdata['sideband_hi']],
                                       **combined_options)
                 combined_options['linestyle'] = self.config['blind_linestyle']
-                combined_options['label'] = f"{label} (blind)"
-                handle_blind = ax.plot(pdata['x'][pdata['blind']], 
-                                       pdata['y'][pdata['blind']],
+                if label_blind is None:
+                    combined_options['label'] = f"{label} (blind)"
+                else:
+                    combined_options['label'] = label_blind
+                x_blind_plus_sideband_edge = [pdata['x'][pdata['sideband_lo']][-1]] + \
+                                             list(pdata['x'][pdata['blind']]) + \
+                                             [pdata['x'][pdata['sideband_hi']][0]]
+                y_blind_plus_sideband_edge = [pdata['y'][pdata['sideband_lo']][-1]] + \
+                                             list(pdata['y'][pdata['blind']]) + \
+                                             [pdata['y'][pdata['sideband_hi']][0]]                
+                handle_blind = ax.plot(x_blind_plus_sideband_edge, 
+                                       y_blind_plus_sideband_edge,
                                        **combined_options)
                 handle = [handle_sblo[0], handle_blind[0]]
             else:
                 handle = ax.plot(pdata['x'], pdata['y'], **combined_options)
                 handle = handle[0]
         else:
-            combined_options = combine_dict(self.styles['errorbar'], plot_options)
-            combined_options['label'] = label
+            
+            combined_options = combine_dict(plot_options, {"label": label})
             if combined_options.get('markersize', None) is None:
                 nbins = len(pdata['x'])
                 combined_options['markersize'] = suggest_markersize(nbins)
-            if not show_errorbar:
-                combined_options['elinewidth'] = 0
-                combined_options['capsize'] = 0
-                combined_options['capthick'] = 0
+                
             if draw_blind:
+                x = pdata['x'][pdata['sideband']]
+                y = pdata['y'][pdata['sideband']]
                 xerr = self._get_selected_err(pdata['xerr'], pdata['sideband'])
                 yerr = self._get_selected_err(pdata['yerr'], pdata['sideband'])
-                handle = ax.errorbar(pdata['x'][pdata['sideband']],
-                                     pdata['y'][pdata['sideband']],
-                                     xerr=xerr,
-                                     yerr=yerr,
-                                     **combined_options)
             else:
-                handle = ax.errorbar(pdata['x'], pdata['y'],
-                                     xerr=pdata['xerr'],
-                                     yerr=pdata['yerr'],
-                                      **combined_options)
+                x = pdata['x']
+                y = pdata['y']
+                xerr = pdata['xerr']
+                yerr = pdata['yerr']
+            if not show_error:
+                error_format = None
+            handle = self._draw_data(ax, x, y, xerr, yerr,
+                                     plot_options=combined_options,
+                                     error_format=error_format,
+                                     error_styles=error_styles)
         return handle
     
-    def process_data(self, data, blind_range:Optional[List[float]]=None):
+    def process_data(self, data, blind_range:Optional[List[float]]=None,
+                     scale_by:Optional[float]=None):
         processed_data = {}
         processed_data['x'] = data['x']
         processed_data['y'] = data['y']
         error_data = self.get_error_data(data)
+        if (scale_by is not None) and (scale_by != 1.):
+            if error_data['yerr'] is not None:
+                raise RuntimeError("scaling is not allowed for data points with error")
+            processed_data['y'] *= scale_by
         processed_data['xerr'] = error_data['xerr']
         processed_data['yerr'] = error_data['yerr']
         if blind_range is not None:
@@ -180,10 +217,47 @@ class PdfDistributionPlot(AbstractPlot):
             processed_data['sideband'] = ~blind
         return processed_data
     
+    def _draw_data(self, ax, x:np.ndarray, y:np.ndarray,
+                   xerr:Optional[np.ndarray]=None,
+                   yerr:Optional[np.ndarray]=None,
+                   plot_options:Optional[Dict]=None,
+                   error_format:Optional[Union[ErrorDisplayFormat, str]]="error_bar",
+                   error_styles:Optional[Dict]=None):
+        combined_options = combine_dict(plot_options, self.styles['errorbar'])
+        if error_format is None:
+            combined_options['elinewidth'] = 0
+            combined_options['capsize'] = 0
+            combined_options['capthick'] = 0
+            handle = ax.errorbar(x, y, xerr=xerr, yerr=yerr,
+                                 **combined_options)
+        elif (error_format == ErrorDisplayFormat.ERRORBAR):
+            combined_options = combine_dict(combined_options, error_styles)
+            handle = ax.errorbar(x, y, xerr=xerr, yerr=yerr,
+                                 **combined_options)
+        elif (error_format == ErrorDisplayFormat.FILL):
+            combined_options['elinewidth'] = 0
+            combined_options['capsize'] = 0
+            combined_options['capthick'] = 0
+            handle = ax.errorbar(x, y, xerr=xerr, yerr=yerr,
+                                 **combined_options)
+            fill_options = combine_dict(self.styles['fill_between'], error_styles)
+            if "zorder" not in fill_options:
+                fill_options["zorder"] = 5
+            if yerr is not None:
+                handle_fill = ax.fill_between(x, y - yerr[0], y + yerr[1],
+                                              **fill_options)
+            else:
+                handle_fill = None
+        else:
+            raise RuntimeError(f"unkown error display format: {error_format}")
+        return handle
+    
     def draw_comparison(self, ax, reference_data, target_data, 
                         xlabel:str="", ylabel:str="",  mode:str="ratio",
-                        plot_options:Optional[Dict]=None, show_errorbar:bool=True, 
+                        plot_options:Optional[Dict]=None, show_error:bool=True, 
                         blind_range:Optional[List[float]]=None,
+                        error_format:Union[ErrorDisplayFormat, str]="error_bar",
+                        error_styles:Optional[Dict]=None,                        
                         draw_ratio_line:bool=True):
         pdata_ref = self.process_data(reference_data, blind_range=blind_range)
         pdata_tgt = self.process_data(target_data, blind_range=blind_range)
@@ -211,31 +285,31 @@ class PdfDistributionPlot(AbstractPlot):
                     pdata_comp['yerr'] = pdata_ref['yerr'] / pdata_tgt['y']
         elif vmode == 1:
             pdata_comp['y'] = pdata_ref['y'] - pdata_tgt['y']
-        combined_options = combine_dict(self.styles['errorbar'], plot_options)
+            
+        combined_options = combine_dict(plot_options)
         if combined_options.get('markersize', None) is None:
             nbins = len(pdata_comp['x'])
             combined_options['markersize'] = suggest_markersize(nbins)
-        if not show_errorbar:
-            combined_options['elinewidth'] = 0
-            combined_options['capsize'] = 0
-            combined_options['capthick'] = 0
+            
         draw_blind = blind_range is not None
         if draw_blind:
             x = pdata_comp['x'][pdata_ref['sideband']]
             y = pdata_comp['y'][pdata_ref['sideband']]
             xerr = self._get_selected_err(pdata_comp['xerr'], pdata_ref['sideband'])
             yerr = self._get_selected_err(pdata_comp['yerr'], pdata_ref['sideband'])
-            handle = ax.errorbar(x, y,
-                                 xerr=xerr,
-                                 yerr=yerr,
-                                 **combined_options)
         else:
             x = pdata_comp['x']
             y = pdata_comp['y']
-            handle = ax.errorbar(x, y,
-                                 xerr=pdata_comp['xerr'],
-                                 yerr=pdata_comp['yerr'],
-                                  **combined_options)
+            xerr = pdata_comp['xerr']
+            yerr = pdata_comp['yerr']
+            
+        if not show_error:
+            error_format = None
+            
+        handle = self._draw_data(ax, x, y, xerr, yerr,
+                                 plot_options=combined_options,
+                                 error_format=error_format,
+                                 error_styles=error_styles)
             
         ylim = list(ax.get_ylim())
         if ylim[0] > np.min(y):
@@ -284,16 +358,27 @@ class PdfDistributionPlot(AbstractPlot):
             "text": text,
             **styles
         }
+        
+    def set_error_options(self, target:str,
+                          display_format:Optional[Union[ErrorDisplayFormat, str]]="errorbar",
+                          styles:Optional[Dict]=None):
+        if display_format is not None:
+            display_format = ErrorDisplayFormat.parse(display_format)
+        self.error_options[target] = {
+            "display_format": display_format,
+            "styles": styles
+        }
 
     def draw(self, xlabel:str="", ylabel:str="",
              targets:Optional[List]=None,
              logx:bool=False, logy:bool=False,
              blind_range:Optional[List[float]]=None,
-             show_errorbar:bool=True, 
-             comparison_options:Optional[str]=None):
+             show_error:bool=True,
+             comparison_options:Optional[Union[Dict, List[Dict]]]=None):
         
-        if comparison_options:
-            ax, ax_ratio = self.draw_frame(ratio_frames, logx=logx, logy=logy)
+        if comparison_options is not None:
+            ax, ax_ratio = self.draw_frame(ratio_frames, logx=logx, logy=logy,
+                                           **self.styles["ratio_frames"])
         else:
             ax = self.draw_frame(logx=logx, logy=logy)
 
@@ -302,11 +387,24 @@ class PdfDistributionPlot(AbstractPlot):
                 continue
             data = self.collective_data[name]
             label = self.label_map.get(name, name)
+            label_blind = self.label_map.get(f"{name}_blind", None)
             plot_options = self.plot_options.get(name, {})
+            scale_by = self.scale_map.get(name, None)
+            if (name not in self.error_options) or \
+               (self.error_options[name]["display_format"] is None):
+                error_format = None
+                error_styles = None
+            else:
+                error_format = self.error_options[name]["display_format"]
+                error_styles = self.error_options[name]["styles"]
             handle = self.draw_single_data(ax, data, label,
+                                           label_blind=label_blind,
                                            blind_range=blind_range,
                                            plot_options=plot_options,
-                                           show_errorbar=show_errorbar)
+                                           show_error=show_error,
+                                           error_format=error_format,
+                                           error_styles=error_styles,
+                                           scale_by=scale_by)
             # case draw blind
             if isinstance(handle, list) and len(handle) == 2:
                 blind_name = f"{name}_blind"
@@ -324,23 +422,38 @@ class PdfDistributionPlot(AbstractPlot):
         handles, labels = self.get_legend_handles_labels()
         ax.legend(handles, labels, **self.styles['legend'])
         
-        if comparison_options:
-            reference = comparison_options['reference']
-            target = comparison_options['target']
-            mode = comparison_options.get('mode', "ratio")
-            ylabel = comparison_options.get('label', "")
-            draw_ratio_line = comparison_options.get('draw_ratio_line', True)
-            plot_options = self.plot_options.get(reference, {})
-            plot_options['color'] = self.colors[reference]
-            self.draw_comparison(ax_ratio,
-                                 self.collective_data[reference],
-                                 self.collective_data[target],
-                                 xlabel=ax.get_xlabel(),
-                                 ylabel=ylabel, mode=mode,
-                                 plot_options=plot_options,
-                                 show_errorbar=show_errorbar,
-                                 blind_range=blind_range,
-                                 draw_ratio_line=draw_ratio_line)
+        if comparison_options is not None:
+            if not isinstance(comparison_options, list):
+                comparison_options = [comparison_options]
+            for options in comparison_options:
+                reference = options['reference']
+                target    = options['target']
+                mode      = options.get('mode', "ratio")
+                ylabel    = options.get('label', "")
+                labelsize = options.get('labelsize', None)
+                draw_ratio_line = options.get('draw_ratio_line', True)
+                plot_options = self.plot_options.get(reference, {})
+                plot_options['color'] = self.colors[reference]
+                if (reference not in self.error_options) or \
+                   (self.error_options[reference]["display_format"] is None):
+                    error_format = None
+                    error_styles = None
+                else:
+                    error_format = self.error_options[reference]["display_format"]
+                    error_styles = self.error_options[reference]["styles"]
+                self.draw_comparison(ax_ratio,
+                                     self.collective_data[reference],
+                                     self.collective_data[target],
+                                     xlabel=ax.get_xlabel(),
+                                     ylabel=ylabel, mode=mode,
+                                     plot_options=plot_options,
+                                     show_error=show_error,
+                                     error_format=error_format,
+                                     error_styles=error_styles,
+                                     blind_range=blind_range,
+                                     draw_ratio_line=draw_ratio_line)
+                if labelsize is not None:
+                    ax_ratio.yaxis.label.set_size(labelsize)                
             ax.set(xlabel=None)
             ax.tick_params(axis="x", labelbottom=False)
 
@@ -348,7 +461,7 @@ class PdfDistributionPlot(AbstractPlot):
             kwargs = combine_dict(self.styles['annotation'], self.annotation)
             ax.annotate(**kwargs)
             
-        if comparison_options:
+        if comparison_options is not None:
             return ax, ax_ratio
         
-        return ax
\ No newline at end of file
+        return ax
