@@ -9,7 +9,7 @@ from matplotlib.lines import Line2D
 from matplotlib.patches import Polygon
 
 from quickstats.plots import AbstractPlot
-from quickstats.plots.template import single_frame, parse_styles, create_transform
+from quickstats.plots.template import single_frame, parse_styles, create_transform, format_axis_ticks
 from quickstats.utils.common_utils import combine_dict
 
 
@@ -45,13 +45,13 @@ class ScoreDistributionPlot(AbstractPlot):
                          analysis_label_options=analysis_label_options,
                          config=config)
     
-    def draw(self, score_name:str="score", weight_name:Optional[str]="weight",
+    def draw(self, column_name:str="score", weight_name:Optional[str]="weight",
              xlabel:str="Score", ylabel:str="Fraction of Events / {bin_width:.2f}",
              boundaries:Optional[List]=None, nbins:int=25, xmin:float=0, xmax:float=1):
         """
         
         Arguments:
-            score_name: string, default = "score"
+            column_name: string, default = "score"
                 Name of the score variable in the dataframe.
             weight_name: (optional) string, default = "weight"
                 If specified, normalize the histogram by the "weight_name" variable
@@ -80,12 +80,12 @@ class ScoreDistributionPlot(AbstractPlot):
                 norm_weights = None
             plot_type = self.plot_options[key].get("type", "hist")
             if plot_type == "hist":
-                y, x, _ = ax.hist(df[score_name], nbins, range=(xmin, xmax),
+                y, x, _ = ax.hist(df[column_name], nbins, range=(xmin, xmax),
                                   weights=norm_weights, **plot_style, zorder=-5)
             elif plot_type == "errorbar":
-                n_data = len(df[score_name])
+                n_data = len(df[column_name])
                 norm_weights = np.ones((n_data,)) / n_data
-                y, bins = np.histogram(df[score_name], nbins, weights=norm_weights)
+                y, bins = np.histogram(df[column_name], nbins, weights=norm_weights)
                 bin_centers  = 0.5*(bins[1:] + bins[:-1])
                 from quickstats.maths.statistics import get_poisson_interval
                 yerr = get_poisson_interval(y * n_data)
@@ -159,7 +159,7 @@ def score_distribution_plot_2D(dfs:Dict[str, pd.DataFrame], hist_options:Dict[st
                             ymax:float=1, xscore_name:str='score', yscore_name:str='score', weight_name:str='weight',
                             xlabel:str='NN Score 1', ylabel:str='NN Score 2', zlabel:str='Fraction of Events / bin',
                             coordinates:Optional[List]=None, plot_styles:Optional[Dict]=None,
-                            analysis_label_options:Optional[Dict]=None):
+                            analysis_label_options:Optional[Dict]=None, show_text=False, density=False):
     styles = parse_styles(plot_styles)
     ax = single_frame(styles=styles, analysis_label_options=analysis_label_options)
     key = list(hist_options.keys())[0]
@@ -170,10 +170,17 @@ def score_distribution_plot_2D(dfs:Dict[str, pd.DataFrame], hist_options:Dict[st
     else:
         log_scale = colors.Normalize()
     combined_df = pd.concat([dfs[sample] for sample in samples], ignore_index = True)
-    norm_weights = combined_df[weight_name]/combined_df[weight_name].sum()
+    norm_weights = combined_df[weight_name]
+    if density:
+        norm_weights /= norm_weights.sum()
     h, x, y, image = ax.hist2d(x=combined_df[xscore_name], y=combined_df[yscore_name], bins=[xbins, ybins],
                         range=[[xmin, xmax], [ymin, ymax]], weights=norm_weights, **hist_style, 
                         norm=log_scale, zorder=-5)
+    if show_text:
+        for i in range(len(y)-1):
+            for j in range(len(x)-1):
+                ax.text((x[j]+x[j+1])/2,(y[i]+y[i+1])/2, h.T[i,j], color="r", ha="center", va="center", fontweight="bold")
+
     if coordinates is not None:
         for coordinate in coordinates:
             ax.plot([i[0] for i in coordinate], [i[1] for i in coordinate], 'r--')
@@ -184,5 +191,9 @@ def score_distribution_plot_2D(dfs:Dict[str, pd.DataFrame], hist_options:Dict[st
     ax.set_xlabel(xlabel, **styles['xlabel'])
     ax.set_ylabel(ylabel, **styles['ylabel'])
     import matplotlib.pyplot as plt
-    plt.colorbar(image, ax=ax, pad=0.02)
+    cbar = plt.colorbar(image, ax=ax, pad=0.02)
+    format_axis_ticks(cbar.ax, x_axis=False, y_axis=True, ytick_styles=styles['ytick'], **styles['z-axis'])
+    cbar.ax.yaxis.set_label_position("right")
+    cbar.ax.yaxis.tick_right()
+    cbar.ax.set_ylabel(zlabel, **styles['zlabel'])
     return ax
