@@ -90,8 +90,14 @@ TEMPLATE_STYLES = {
             "capthick": 1,
             "zorder": 1
         },
+        'fill_between': {
+        },
         'legend':{
             "fontsize": 20
+        },
+        'ratio_frames':{
+            'height_ratios': (3, 1),
+            'hspace': 0.07            
         }
     }
 }
@@ -113,6 +119,14 @@ AXIS_LOCATOR_MAPS = {
     'maxn': MaxNLocator
 }
 
+def handle_has_label(handle):
+    try:
+        label = handle.get_label()
+        has_label = (label and not label.startswith('_'))
+    except:
+        has_label = False
+    return has_label
+
 def parse_styles(styles:Optional[Union[Dict, str]]=None):
     default_styles = deepcopy(TEMPLATE_STYLES['default'])
     if styles is None:
@@ -265,7 +279,8 @@ def format_axis_ticks(ax, x_axis=True, y_axis=True, major_length:int=16, minor_l
     if offsetlabelsize is None:
         offsetlabelsize = labelsize
     # update the offset text
-    plt.tight_layout()
+    if plt.gca().__class__.__name__ != "AxesSubplot":
+        plt.tight_layout()
     if ax.xaxis.get_offset_text().get_text():
         ax.xaxis.get_offset_text().set_fontsize(offsetlabelsize)
         ax.xaxis.labelpad = ax.xaxis.labelpad + ax.xaxis.get_offset_text().get_fontsize()
