@@ -54,7 +54,7 @@ class UpperLimitBenchmarkPlot(AbstractPlot):
                  analysis_label_options:Optional[Union[Dict, str]]=None):
         """
         Args:
-            benchmar_df: pandas.DataFrame
+            benchmark_df: pandas.DataFrame
                 dataframe with columns ("-2", "-1", "0", "1", "2", "obs") representing
                 the corresponding limit level and rows indexed by the benchmark names
         """
