@@ -205,6 +205,9 @@ def format_delimiter_enclosed_text(text:str, delimiter:str, width:int=10, indent
         result += f"{indent_str} {line}{text.center(2*width+2)}{line}\n"
     result += f"{indent_str} {full_line}" + "\n"*lower_margin
     return result
+
+def insert_periodic_substr(s:str, every:int=64, substr:str='\n'):
+    return substr.join(s[i:i+every] for i in range(0, len(s), every))
         
 
 def filter_by_wildcards(targets:List, conditions:Optional[Union[str, List]]=None, exclusion:bool=False):
