@@ -38,9 +38,8 @@ def array2root(array_data:Dict[str, np.ndarray], fname:str, tree_name:str,
         from packaging import version
         uproot_version = uproot.__version__
         if version.parse(uproot_version) < version.parse("4.2.0"):
-            file = uproot.writing.writable.recreate(fname)
-        else:
-            file = uproot.recreate(fname)
+            raise RuntimeError("uproot version too old (<4.2.0)")
+        file = uproot.recreate(fname)
         file[tree_name] = array_data
         file.close()
     else:
@@ -71,6 +70,10 @@ def get_default_library():
     try:
         import uproot
         has_uproot = True
+        uproot_version = uproot.__version__
+        if version.parse(uproot_version) < version.parse("4.2.0"):
+            print("WARNING: uproot version too old (<4.2.0), will switch to using ROOT instead")
+            has_uproot = False
     except ImportError:
         has_uproot = False
     if has_uproot:
